;converting all my screen stuff to mode x
;in 4gw leave es, ds alone
;bp, sp =>ebp, esp
;[ebp+8], [ebp+12]
;ax dx bx cx
;*******************************
stackseg        segment stack 'stack'
        db      512 dup (?)
stackseg        ends
;*******************************
public  initx_          ;initialize mode x
public  mode80_         ;go back to text mode
public  psetx_          ;put a pixel on the screen
public  flipx_          ;flip to a new page 
public  putx_           ;block copy a pic to the screen
public  putranx_        ;put a picture with transparent on screen
public  paintx_         ;fill the screen with a color
public  keypress_       ;return nozero if a key has been pressed
public  palette_        ;set the palette
public  putcan_         ;put a pic. with transparent on canvas
public  clearcan_       ;clear the canvas
public  paintcan_       ;fill the screen with the canvas
;***********************************

;*******************************
sc_index        equ 03c4h       ;sequence controler index
crtc_index      equ 03d4h       ;crt controller index
crtc_offset     equ 13h         ;crt offset reg
misc_output     equ 03c2h       ;misc output register
screen_seg      equ 0a0000h     ;screen segment
input_status_1  equ 03dah       ;input status 1 register
start_adres_hi  equ 0ch         ;start address high byte
start_adres_low equ 0dh         ;start address low byte

map_mask        equ 02h         ;index in sc of map mask regester
screen_width    equ 90          ;width of screen in bytes for each scan line

;*******************************

dataseg segment 'data'

crtparms label  word
        dw 00d06h 
        dw 03e07h 
        dw 04109h 
        dw 0ea10h 
        dw 0ac11h 
        dw 0df12h 
        dw 00014h 
        dw 0e715h 
        dw 00616h 
        dw 0e317h 
crt_parm_len    equ (($-crtparms)/2)        

curplane db ?           ;which plane were writing to now
pic_add dd ?            ;address of top left pixel of picture 
pwidth  dw ?            ;width of the picture
pheight dw ?            ;height of the picture
nline   dd ?            ;offset to the next line when drawing picture
dataseg ends

;*******************************
;macros
;***********************************
before  macro
        push ebp      ;Save the base pointer
        mov ebp, esp          ;Copy the stack pointer
        push esi
        push edi
        endm

; Postfix -- Returns the pointers to the condition they were in when
;               the C program called the routine

after   macro
        pop edi
        pop esi
        mov esp, ebp   ;Restore the stack pointer
        pop ebp
             ;Restore the base pointer
        ret             ;return from subroutine
        endm

setreg  macro   p1, p2
        mov dx, p1
        mov ah, al
        mov al, p2
        out dx, ax
        endm

;***********************************

_cdseg  segment byte public 'CODE'      ;Program segment begins here

;set 320*240*256 mode
;no arguements
initx_  proc near
        before
        mov ax, 13h
        int 10h
        
        mov dx, sc_index           ;sequence controller index
        mov ax, 0604h
        out dx, ax      ;disable chain4
        
        mov ax, 0100h
        out dx, ax      ;synchronous reset for safety

        mov dx, misc_output   ;miscellaneous output regester
        mov al, 0e3h
        out dx, al      ;select 25mhz dot clack and 60 hz scanning       

        mov dx, sc_index
        mov ax, 0300h
        out dx, ax      ;undo reset (restart sequencer)

        mov dx, crtc_index      ;reprogram crt controller
        mov al, 11h
        out dx, al      ;protect bit
        inc dx
        in al, dx
        and al, 7fh
        out dx, al
        dec dx
        cld
        mov esi, offset crtparms        ;crt parameter table
        mov cx, crt_parm_len            ;# of table entries
setcrtp:
        lodsw
        out dx, ax
        loop setcrtp

        mov dx, sc_index       ;clear the screen 16 pix at a time
        mov ax, 0f02h
        out dx, ax      ;enable writes to all four planes
        mov edi, screen_seg
        xor eax, eax
        mov cx, 4000h   ;number of writes needed
        rep stosd       ;store the stuff
;***********************************
        ;test section-seting logical screen width
        mov al, screen_width/2
        setreg crtc_index, crtc_offset
;***********************************
        after
        ret
initx_   endp
;---------
;set text mode
;no arguements
mode80_ proc near
        mov ax, 03h
        int 10h
        ret
mode80_ endp
;---------
;put a pixel on the screen
; eax = x
; edx = y
; ebx = color
; ecx =offset of page we want to write to
psetx_   proc near
        before
     
        mov edi, screen_seg  ;vga mem
        add edi, ecx            ;go to our page
        
        mov cl, al      ;save the x value
        
        shr eax, 1       ;add the x value divided by 4
        shr eax, 1
        add edi, eax

        mov eax, edx
        mov dx, screen_width
        mul dx
        add edi, eax     ;add the y value

        and cl, 011b    ;see which plane (x mod 4)
        mov ax, 0100h+map_mask  ;al index in sc of map mask 
        shl ah, cl         ;select the plane
        mov dx, sc_index        
        out dx, ax
        
        mov [edi], bl

        after
        ret
psetx_   endp
;---------
;ax contains memory to point to
flipx_  proc near
        before
        mov bl, start_adres_low       ;load stuff needed to flip
        mov bh, al
        mov cl, start_adres_hi
        mov ch, ah
        mov dx, input_status_1
;wait for display enable inactive (retrace?)
waitDE:
        in al, dx
        test al, 01h
        jnz waitDE

        mov dx, crtc_index
        mov ax, bx
        out dx, ax      ;start address high
        mov ax, cx
        out dx, ax      ;start address low
;wait for vertical sync
        mov dx, input_status_1
waitVS:
        in al, dx
        test al, 08h
        jz waitVS               ;vertical sync is active

        after
flipx_  endp
;---------
;---------
;paint the whole screen 
; eax = page
; edx = color  (actually al)
paintx_ proc near
        before
        
        mov edi, screen_seg
        add edi, eax    ;go to the page to paint
        mov dh, dl      ;put color in dh
        mov cx, dx      ;save color
        mov dx, sc_index       ;clear the screen 16 pix at a time
        mov ax, 0f02h
        out dx, ax      ;enable writes to all four planes
        mov ax, cx
        mov bx, ax      ;save the color
        mov edx, 10000h
        mul edx
        mov ax, bx

        mov cx, 4000h  ;number of writes needed
        rep stosd       ;store the stuff

        after
        ret
paintx_ endp

;---------
;---------
;eax x coordinate
;edx y coordinate
;ebx picture
;ecx page
putx_   proc near
        before
        
        mov edi, screen_seg     ;vga mem
        add edi, ecx            ;go to our page
        mov esi, ebx            ;the image is now in esi
        
        mov cl, al      ;save the x value
        
        shr eax, 1       ;add the x value divided by 4
        shr eax, 1
        add edi, eax

        mov eax, edx
        mov dx, screen_width
        mul dx
        add edi, eax     ;add the y value

        and cl, 011b    ;see which plane (x mod 4)
        mov ax, 0100h+map_mask  ;al index in sc of map mask 
        shl ah, cl         ;select the plane
        ;
        mov pic_add, edi        ;save the top left corner
        mov curplane, ah        ;save the current plane
        ;
        mov dx, sc_index        
        out dx, ax
        
        lodsw                   ;load the width of the 1/4 image
        mov pwidth, ax          ;save
        mov ebx, screen_width    ;calculate the jump to the next line 
        sub bx, ax
        mov nline, ebx           ;save jump to the next line
        
        lodsw                   ;load the picture height
        mov pheight, ax

        mov bl, 4;
drawplane:        
        ;all set, draw the first plane of the picture
        mov dx, pheight
next_line:
        mov cx, pwidth          ;store this line
        shr cx, 1
        jnc odd
        movsb
odd:    rep movsw
        
        add edi, nline          ;go to the next line
        dec dx
        jnz next_line

        dec bl
        jz thatsall         ;end if no other plane to draw
        ;swithc to next plane
        mov edi, pic_add
        mov ah, curplane
        shl ah, 1
        cmp ah, 010000b
        jne noinc
        ;increment edi, set ah to 1
        mov ah, 1
        inc edi

noinc:          ;dont have to increment edi
        ;go to new plane
        mov al, map_mask  ;al index in sc of map mask 
        ;
        mov pic_add, edi        ;save the new offset and plane
        mov curplane, ah        
        ;
        mov dx, sc_index        
        out dx, ax
        jmp drawplane
thatsall:
        
        after
putx_   endp

;---------

;---------
;eax x coordinate
;edx y coordinate
;ebx picture
;ecx page
putranx_ proc near
        before
        
        mov edi, screen_seg     ;vga mem
        add edi, ecx            ;go to our page
        mov esi, ebx            ;the image is now in esi
        
        mov cl, al      ;save the x value
        
        shr eax, 1       ;add the x value divided by 4
        shr eax, 1
        add edi, eax

        mov eax, edx
        mov dx, screen_width
        mul dx
        add edi, eax     ;add the y value

        and cl, 011b    ;see which plane (x mod 4)
        mov ax, 0100h+map_mask  ;al index in sc of map mask 
        shl ah, cl         ;select the plane
        ;
        mov pic_add, edi        ;save the top left corner
        mov curplane, ah        ;save the current plane
        ;
        mov dx, sc_index        
        out dx, ax
        
        lodsw                   ;load the width of the 1/4 image
        mov pwidth, ax          ;save
        mov ebx, screen_width    ;calculate the jump to the next line 
        sub bx, ax
        mov nline, ebx           ;save jump to the next line
        
        lodsw                   ;load the picture height
        mov pheight, ax

        mov bl, 4;
drawplanet:        
        ;all set, draw the first plane of the picture
        mov dx, pheight
next_linet:
        mov cx, pwidth          ;store this line
next_col:        
        lodsb
        cmp al, 255
        jne storeit
        inc edi
        jmp dontstoreit
storeit:
        stosb
dontstoreit:        
        dec cx
        jnz next_col

        add edi, nline          ;go to the next line
        dec dx
        jnz next_linet

        dec bl
        jz thatsallt         ;end if no other plane to draw
        ;switch to next plane
        mov edi, pic_add
        mov ah, curplane
        shl ah, 1
        cmp ah, 010000b
        jne noinct
        ;increment edi, set ah to 1
        mov ah, 1
        inc edi

noinct:          ;dont have to increment edi
        ;go to new plane
        mov al, map_mask  ;al index in sc of map mask 
        ;
        mov pic_add, edi        ;save the new offset and plane
        mov curplane, ah        
        ;
        mov dx, sc_index        
        out dx, ax
        jmp drawplanet
thatsallt:
        
        after
putranx_ endp

;---------


;---------
;paint the whole screen with the canvas
; eax = canvas
paintcan_  proc near
        before
        mov esi, eax
        mov edi, 0a0000h  ;vga mem
        mov ecx, 16000
        ;no flicker wait for screen retrace
vertic:
        mov dx, 03dah
        in al, dx
        test al, 8
        jz vertic
        rep movsd
        after
        ret
paintcan_  endp

;---------
;---------
;clear the canvas
; eax = canvas
clearcan_  proc near
        before
        mov edi, eax
        mov ecx, 16000
        mov eax, 0
        rep stosd
        after
        ret
clearcan_  endp

;---------
;---------
;ebx picture
;eax x coordinate
;edx y coordinate
;ecx canvas
putcan_ proc near
        before
        mov esi, ebx
        mov edi, ecx
        add edi, eax        
        mov eax, edx
        mov ecx, 320
        mul ecx
        add edi, eax

        mov eax, 0  
        ;add si, 4
        lodsw
        mov ebx, eax
        lodsw
        mov ecx, eax
        inc ecx
        inc ebx


        mov edx, 320
        sub edx, ebx

his:
        push ecx
        mov ecx, ebx
problem:
        lodsb
        cmp al, 255     ;see if transparrent
        je clearer
        mov [edi], al
clearer:
        inc edi
        dec ecx
        jnz problem        
        ;rep movsb

        pop ecx
        add edi, edx
        dec ecx
        jnz his

        after
putcan_ endp

;---------
;returns nonzero for keypressed
keypress_ proc near
        before
        mov eax, 0
        mov ah, 0bh
        int 21h
        mov ah, 0
        after
keypress_ endp    
;---------
;eax pallete

palette_  proc near
        ;lets move some color
        before
        mov esi, eax           ;offset of pallette
        mov cx, 256
well:
        mov dx, 3c8h
        mov ax, 256
        sub ax, cx
        out dx, al
        mov dx, 3c9h
        lodsb
        out dx, al
        lodsb
        out dx, al
        lodsb
        out dx, al
        dec cx
        jnz well
        after
        ret
palette_        endp
;--------------
_cdseg          ends                   ;End code segment
        end                    ;End program
